'' =================================================================================================
''
'' SplashBoot v0.1 by Alessandro De Luca
''
''
'' based on:
''
''   1. SDLoader 1.0 by Brian Riley
''     various parts based upon work/ideas from:
''       Matthew Cornelisse
''       Nick McClanahan
''       Jon McPhalen
''     and a lot of work by Mike Green on the underlying
''     hardware drivers as well as help converting to the Femto
''     based sd/spi/i2c hardware handlers
''
''   2. 2BitBitMapDemo (TV version) by Raymond Allen
''
''   3. 2BitBitMapDemo (VGA version) by Raymond Allen
''
''
'' target demos:
''
''   1. BlackJack by Raymond Allen
''       available at: http://www.rayslogic.com/propeller/Programming/Programming.htm
''     renamed as "BOOT_TVC.BIN"
''
''   2. BlackJack by Raymond Allen (modified for VGA)
''     renamed as "BOOT_VGA.BIN"
''
''   3. Lerner 1.0 by Dave Hein
''     renamed as "BOOT_SER.BIN"
''
''-------------------------------------------------------------------------------


''-------------------------------------------------------------------------------

'#define DEBUG

CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000       '80 MHz

  PAGESIZE = 32

'  MYBOARD = hwd#HW_BOARD_DEMO
  MYBOARD = hwd#HW_BOARD_PROTO
'  MYBOARD = hwd#HW_BOARD_RAMBLADE

  SDBASEPIN = 12  'FIXME: should be autoselected depending on MYBOARD!

  LO_EEPROM = $0000           ' based upon 24LC512 (64KB)
  HI_EEPROM = $8000

CON
  nuchars = 176 'number of custom interleaved characters
  '!!!!! you must put the correct # here for alignment later

  tvc_cols = 20
  tvc_rows = 14
  tvc_screensize = tvc_cols * tvc_rows
  tvc_count = 14

  vga_cols = 32
  vga_rows = 16
  vga_screensize = vga_cols * vga_rows
  vga_count = 21

VAR
  byte buffer[PAGESIZE]
  long maincog
  long ss[64]                ' Stack for watchdog timer
  long ioControl[2]

VAR
  long  col, row, color, flag
  word  screen[vga_screensize]
  long  colors[8 * 2]
  long  tv_status     '0/1/2 = off/invisible/visible              read-only   (14 longs)
  long  tv_enable     '0/non-0 = off/on                           write-only
  long  tv_pins       '%pppmmmm = pin group, pin group mode       write-only
  long  tv_mode       '%tccip = tile,chroma,interlace,ntsc/pal    write-only
  long  tv_screen     'pointer to screen (words)                  write-only
  long  tv_colors     'pointer to colors (longs)                  write-only
  long  tv_ht         'horizontal tiles                           write-only
  long  tv_vt         'vertical tiles                             write-only
  long  tv_hx         'horizontal tile expansion                  write-only
  long  tv_vx         'vertical tile expansion                    write-only
  long  tv_ho         'horizontal offset                          write-only
  long  tv_vo         'vertical offset                            write-only
  long  tv_broadcast  'broadcast frequency (Hz)                   write-only
  long  tv_auralcog   'aural fm cog                               write-only
  word user_charbase  'for custom characters

VAR
'  long  col, row, color
'  word  screen[tvc_screensize]
  long  boxcolor,ptr,xcols
  long  vga_status      'status: off/visible/invisible  read-only       (21 contiguous longs)
  long  vga_enable      'enable: off/on                 write-only
  long  vga_pins        'pins: byte(2),topbit(3)        write-only
  long  vga_mode        'mode: interlace,hpol,vpol      write-only
  long  vga_videobase   'video base @word               write-only
  long  vga_colorbase   'color base @long               write-only
  long  vga_hc          'horizontal cells               write-only
  long  vga_vc          'vertical cells                 write-only
  long  vga_hx          'horizontal cell expansion      write-only
  long  vga_vx          'vertical cell expansion        write-only
  long  vga_ho          'horizontal offset              write-only
  long  vga_vo          'vertical offset                write-only
  long  vga_hd          'horizontal display pixels      write-only
  long  vga_hf          'horizontal front-porch pixels  write-only
  long  vga_hs          'horizontal sync pixels         write-only
  long  vga_hb          'horizontal back-porch pixels   write-only
  long  vga_vd          'vertical display lines         write-only
  long  vga_vf          'vertical front-porch lines     write-only
  long  vga_vs          'vertical sync lines            write-only
  long  vga_vb          'vertical back-porch lines      write-only
  long  vga_rate        'pixel rate (Hz)                write-only
'  word user_charbase    'for custom characters

OBJ
  sdc : "fsrwFemto"        ' Has different mount call
  spi : "sdspiFemto"       ' SPIN program loader and support routines
  hwd : "hwdetect"
#ifdef DEBUG
  ser : "FullDuplexSerial"
#endif
  tvc : "tv"
  vga : "vga"

DAT
bootfile      byte      "BOOT_xxx.BIN",0
              byte      0,0,0   'PAD

PUB start | wdcog, conio_id
#ifdef DEBUG
  ser.start(31, 30, 0, 115200)
  waitcnt(cnt+clkfreq*3)        ' wait for 3 seconds
#endif
  repeat
#ifdef DEBUG
    ser.str(string("Checking for console mode..."))
#endif
    conio_id := hwd.ConsoleDetect(MYBOARD)
    case conio_id
      hwd#HW_CONIO_VGAKBD:
#ifdef DEBUG
        ser.str(string("using VGA screen and keyboard!",13,10))
#else
        ShowSplashVGA(16, 7)
#endif
        setupname(string("VGA"))
      hwd#HW_CONIO_TVCKBD:
#ifdef DEBUG
        ser.str(string("using TV screen and keyboard!",13,10))
#else
        ShowSplashTVC(16, 7)
#endif
        setupname(string("TVC"))
      hwd#HW_CONIO_1PINVK:
#ifdef DEBUG
        ser.str(string("using 1pin TV & KB!",13,10))
#endif
        setupname(string("1VK"))
      hwd#HW_CONIO_SERIAL:
#ifdef DEBUG
        ser.str(string("using serial link!",13,10))
#endif
        setupname(string("SER"))
      other:
#ifdef DEBUG
        ser.str(string("ERROR: unknown console mode!",13,10))
#endif
        setupname(string("SIM"))
    maincog := cogid              ' get cog# of current cog
    sdc.start(@iocontrol)         ' start fsrw, point to IO control block
    wdcog:=cognew(wd,@ss)         ' start watchdog
    if \sdc.mount(SDBASEPIN+0,SDBASEPIN+1,SDBASEPIN+2,SDBASEPIN+3) == 0
'      cogstop(wdcog)              ' stop watchdog
#ifdef DEBUG
      ser.str(string("booting "))
      ser.str(@bootfile)
      ser.str(string("..."))
#endif
      if \sdc.popen(@bootfile, "r") == 0
        sdc.bootSDCard            ' this loads the current open file from SD card and runs it

'    spi.bootEEPROM(spi#bootAddr + $8000) ' load from 2nd 32K of EEPROM
    waitcnt(cnt)                'just let the watchdog expire!

PRI wd
  waitcnt(cnt+clkfreq*15)       ' wait for 15 seconds
  cogstop(maincog)              ' stop hung main cog
  spi.bootEEPROM(spi#bootAddr + $8000) ' load from 2nd 32K of EEPROM

PRI setupname(strptr)
  bootfile[ 5] := byte[strptr+0]
  bootfile[ 6] := byte[strptr+1]
  bootfile[ 7] := byte[strptr+2]

PUB ShowSplashTVC(basepin, pauselen) : okay
  xcols := tvc_cols
  '64 byte align the user characters
  user_charbase := @uchar & $FFC0 'destination
  'user_charbase_offset := user_charbase-@uchar
  longmove(user_charbase,@uchar,16*nuchars)
  ''--- Start terminal - starts a cog
  '' returns false if no cog available
  longmove(@tv_status, @tvc_params, tvc_count)
  tv_pins := (basepin & $38) << 1 | (basepin & 4 == 4) & %0101
  tv_screen := @screen
  tv_colors := @colors
  okay := tvc.start(@tv_status)
  wordfill(@screen, $220, tvc_screensize)
  'show Prop logo on dk. blue background
  colors[ 0] := $05_2A_05_2A 'NOTE:  last color here (LSB) is the overall background
  colors[ 1] := $05_05_2A_2A
  colors[ 9] := $04_07_02_2A 'NOTE:  color 1 on right side (LSB), color 4 on left side (MSB)
  colors[10] := $AB_AC_02_2A
  colors[11] := $AC_8E_B8_2A
  'logo is actually 3 seperate bitmaps
  Bitmap2Bit(@PropUpper,  2,1,16,5,9)  'address,x,y,width,height,color
  Bitmap2Bit(@PropMiddle, 2,6,16,4,10)
  Bitmap2Bit(@PropLower,  2,10,16,2,11)
  waitcnt(cnt+clkfreq*pauselen)
  ''--- Stop terminal - frees a cog
  tvc.stop

PUB ShowSplashVGA(basepin, pauselen) : okay
  xcols := vga_cols
  '64 byte align the user characters
  user_charbase := @uchar & $FFC0 'destination
  'user_charbase_offset := user_charbase-@uchar
  longmove(user_charbase,@uchar,16*nuchars)
  wordfill(@screen, $220, vga_screensize)
  longmove(@vga_status, @vga_params, vga_count)
  vga_pins := %010111
  vga_videobase := @screen
  vga_colorbase := @vga_colors
  ''--- Start terminal - starts a cog
  result := vga.start(@vga_status)
  'show Prop logo
  'logo is actually 3 seperate bitmaps
  Bitmap2Bit(@PropUpper,8,1,16,5,21)  'address,x,y,width,height,color
  Bitmap2Bit(@PropMiddle,8,6,16,4,22)
  Bitmap2Bit(@PropLower,8,10,16,2,23)
  waitcnt(cnt+clkfreq*pauselen)
  ''--- Stop terminal - frees a cog
  vga.stop


PRI Bitmap2Bit(pBitmap, xPos, yPos, xSize, ySize, clr)|c,i,j,BmpAddress
  row:=yPos
  col:=xPos
  c:=0
  BmpAddress:=pBitmap+user_charbase-@uchar
  repeat j from 0 to (ySize-1)
    repeat i from 0 to (xSize-1)
      screen[row * xcols + col] := (clr) << 10 + BmpAddress>>6 +c
      c++
      col++
    row++
    col:=xPos
{
PRI Bitmap2Bit(pBitmap, xPos, yPos, xSize, ySize, clr)|c,i,j,BmpAddress
  row:=yPos
  col:=xPos
  c:=0
  BmpAddress:=pBitmap+user_charbase-@uchar
  repeat j from 0 to (ySize-1)
    repeat i from 0 to (xSize-1)
      screen[row * vga_cols + col] := (clr) << 10 + BmpAddress>>6 +c
      'Print2Bit(c,clr,pBitmap)
      c++
      col++
    row++
    col:=xPos
}
DAT
tvc_params              long    0               'status
                        long    1               'enable
                        long    0               'pins
                        long    %00010          'mode        (%00000 NON-INTERLACED, %00010 FOR INTERLACED)
                        long    0               'screen
                        long    0               'colors
                        long    tvc_cols        'hc
                        long    tvc_rows        'vc
                        long    9               'hx
                        long    2               'vx          (1 NON-INTERLACED, 2 FOR INTERLACED)
                        long    0               'ho
                        long    0               'vo
                        long    0               'broadcast
                        long    0               'auralcog
                        '       fore   back
                        '       color  color
tvc_colors              byte    $07,   $0A    '0    white / dark blue
                        byte    $07,   $BB    '1    white / red
                        byte    $9E,   $9B    '2   yellow / brown
                        byte    $04,   $07    '3     grey / white
                        byte    $3D,   $3B    '4     cyan / dark cyan
                        byte    $6B,   $6E    '5    green / gray-green
                        byte    $BB,   $CE    '6      red / pink
                        byte    $3C,   $0A    '7     cyan / blue

DAT
vga_params              long    0               'status
                        long    1               'enable
                        long    %010_111        'pins
                        long    %011            'mode
                        long    0               'videobase
                        long    0               'colorbase
                        long    vga_cols        'hc
                        long    vga_rows        'vc
                        long    1               'hx
                        long    1               'vx
                        long    0               'ho
                        long    0               'vo
                        long    512             'hd
                        long    16              'hf
                        long    96              'hs
                        long    48              'hb
                        long    380             'vd
                        long    11              'vf
                        long    2               'vs
                        long    31              'vb
                        long    20_000_000      'rate
vga_colors
                        long    $00040004       'dk.blue/black
                        long    $00000404
                        long    $08A808A8       'green
                        long    $0808A8A8
                        long    $50005000       'blue
                        long    $50500000
                        long    $FC00FC00       'white
                        long    $FCFC0000
                        long    $FF80FF80       'red/white
                        long    $FFFF8080
                        long    $FF20FF20       'green/white
                        long    $FFFF2020
                        long    $FF28FF28       'cyan/white
                        long    $FFFF2828
                        long    $00A800A8       'grey/black
                        long    $0000A8A8
                        long    $C0408080       'redbox
spcl
                        long    $30100020       'greenbox
                        long    $3C142828       'cyanbox
                        long    $FC54A8A8       'greybox
                        long    $3C14FF28       'cyanbox+underscore
bitmapcolors
                        long    %%1110_3330_0000_0010                 '21    dk.gray, white, black, dk.blue
                        long    %%2000_3000_0000_0010                 '22    dk.red, red, black, dk.blue
                        long    %%3100_3300_0230_0010                 '23    orange, yellow, cyan, dk.blue
                        long    %%0000_1110_2220_3330                 '24    orange, yellow, cyan, dk.blue
altscreen
                        byte    $15,$6E,$20,$14,$20,$B1,$6D,$16,$20,$20,$20,$20,$20,$20,$20,$20
                        byte    $20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$5F,$5F,$5F,$5F,$5F,$20
                        byte    $20,$81,$82,$86,$87,$8B,$80,$8b,$80,$8B,$20,$20,$AF,$BD,$BE,$9B
                        byte    $7F,$9B,$A6,$A8,$AA,$20,$20,$17,$20,$20,$53,$54,$41,$52,$54,$20
                        byte    $20,$20,$20,$20,$20,$20,$20,$20,$20,$17,$20,$20,$20,$20,$20,$AC
                        byte    $20,$AC,$20,$20,$20,$20,$1D,$1E,$20,$20,$20,$20,$20,$20,$20,$20
                        byte    $20,$20,$42,$49,$41,$53,$20,$A0,$1A,$1B,$20,$20,$20,$20,$20,$18
                        byte    $20,$18,$20,$20,$20,$20,$20,$91,$20,$23,$24,$38,$30,$30,$30,$20
                        byte    $3A,$3D,$20,$20,$20,$20,$20,$20,$20,$99,$BB,$20,$56,$62,$20,$20
                        byte    $20,$20,$20,$20,$20,$20,$20,$BC,$20,$23,$25,$31,$30,$30,$31,$20
                        byte    $20,$50,$30,$20,$BA,$BB,$90,$94,$20,$B6,$20,$31,$30,$30,$6B,$13
                        byte    $20,$17,$20,$20,$20,$20,$20,$18,$20,$70,$5B,$69,$5D,$20,$20,$20
                        byte    $20,$20,$20,$20,$20,$20,$20,$20,$20,$18,$20,$20,$20,$20,$20,$20
                        byte    $AB,$AC,$AD,$31,$30,$B5,$46,$20,$20,$20,$20,$20,$20,$20,$20,$20
                        byte    $12,$10,$13,$41,$2F,$44,$20,$20,$20,$20,$58,$B2,$2D,$20,$31,$20
                        byte    $20,$18,$20,$20,$20,$20,$20,$20,$63,$61,$74,$5F,$64,$6F,$70,$65

DAT
padding LONG  7[16] 'alignment padding for the following user defined characters

uchar long

PropUpper long
'Prop_Upper is 16x5
file "Prop_Upper.dat"

'Prop_Middle is 16x4
PropMiddle long
file "Prop_Middle2.dat"

'Prop_Lower is 16x2
PropLower long
file "Prop_Lower.dat"

CON'-------------------------------------------------------------------
{{
                            TERMS OF USE: MIT License

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
}}

