'******************************************************************************
' Hardware detection routines
' Author: Alessandro De Luca
' Copyright (c) 2011
' See end of file for terms of use.
'******************************************************************************
'******************************************************************************
' Revison History
' v0.1 - 5/1/2011 First release
'******************************************************************************
{{
}}
CON
  HW_BOARD_UNKNOWN  = 0
  HW_BOARD_DEMO     = 1
  HW_BOARD_PROTO    = 2
  HW_BOARD_HYDRA    = 3
  HW_BOARD_RAMBLADE = 4
  HW_BOARD_PROPCADE = 5
  HW_BOARD_PROPASC  = 6
  HW_BOARD_C3       = 7

  HW_CONIO_FAILED = $FF         'no console, mute output (?)

  HW_CONIO_SERIAL = $00         'default, console to std serial
  HW_CONIO_SIMSER = $01         'spin simulator serial
'  HW_CONIO_PCTERM = $02         'PropTerm PC link - TBD!
'  HW_CONIO_PROXYC = $03         'Catalina Proxy Client - TBD!

  HW_CONIO_1PINVK = $10         '1pin TV and KB (TODO: hack for continuous baudrate detection and remove startup timing?)
'  HW_CONIO_1PGRAY = $11         '1pin TV using grayscale PWM 256x192-4bpp, and 1pin KB - TBD!

  HW_CONIO_VGAKBD = $20         'single COG VGA + keyboard
'  HW_CONIO_VGAKNM = $21         'single COG VGA + keyboard AND mouse - TBD!
'  HW_CONIO_VGAKNP = $22         'single COG VGA + keyboard AND Wii Pad! (more to come later) - TBD!

  HW_CONIO_TVCKBD = $30         'single COG TVC + keyboard
'  HW_CONIO_TVCKNM = $31         'single COG TVC + keyboard AND mouse - TBD!
'  HW_CONIO_TVCKNP = $32         'single COG TVC + keyboard AND Wii Pad! (more to come later) - TBD!

{
  SpinSim:                      Sim Serial  (HW_CONIO_SIMSER) => FIXED FUNCTION!
                                no autodetect!

  Proto/Drac autodetect:        TVA adapter (HW_CONIO_TVCKBD) -> VGA monitor (HW_CONIO_VGAKBD) -> Serial Port (HW_CONIO_SERIAL)
                                VGA (16..23), TVC (16..19), SER (30,31)

  DemoBoard autodetect:         TVA adapter (HW_CONIO_TVCKBD) -> VGA monitor (HW_CONIO_VGAKBD) -> Serial Port (HW_CONIO_SERIAL)
                                VGA (16..23), TVC (12..15), SER (30,31)

  RamBlade (bare):              1pin Vid/KB (HW_CONIO_1PINVK) -> Serial port (HW_CONIO_SERIAL)
                                1VK (22,23), SER (22,23)

  NOTE 1: sub-variants must be forced!
  NOTE 2: all keyboards MUST get TFS mod, and maybe more reset modes/buttons!
}
  CHKDELAY   = 1                'check routines internal small delay between ops, in msec
  PREBIASLEN = 65536            'prebias burst lenght (done at spin max toggling speed)
  RBBURSTLEN = 4096

PUB Null
  return -1

PUB ConsoleDetect(BoardID) | rval
{
  Secondary autodetection:
    P12-P14 has TV circuit?  => using DemoBoard (or DB clone)
    P20-P23 has VGA circuit? => using ProtoBoard (or DracBlade)
    otherwise                => using RamBlade (or unknown board)
}
  case BoardID
    HW_BOARD_DEMO:
      rval := AutoDetect_Demo
    HW_BOARD_PROTO:
      rval := AutoDetect_Proto
    HW_BOARD_RAMBLADE:
      rval := AutoDetect_RamBlade
    other:
      rval := AutoDetect_Unknown
  return rval

PRI msleep(ms) | onems
  onems := clkfreq/1_000
  waitcnt(cnt+(ms*onems))

PRI HW_Discharge(BasePin, Delay, Mask) | HighPin
  HighPin := BasePin + 3
  OUTA[HighPin..BasePin] := %0000
  DIRA[HighPin..BasePin] := Mask
  msleep(delay)
  DIRA[HighPin..BasePin] := %0000

PRI HW_PreBias(BasePin, NumRep, Mask) | HighPin, oval
  HighPin := BasePin + 3
  OUTA[HighPin..BasePin] := %0000
  DIRA[HighPin..BasePin] := Mask
  oval := %0101
  repeat NumRep
    OUTA[HighPin..BasePin] := oval
    oval := oval ^ %1111
  DIRA[HighPin..BasePin] := %0000

PRI HW_Release
' make sure we don't left anything driving the pins!
  DIRA[31..0] := 0
  OUTA[31..0] := 0

PRI AutoDetect_Demo | rval, tval, oval
{
  Auto detect for DemoBoard
    we already know we have VGA at P16 and TV circuit on P12-P14
    TV test must ONLY check for a cable, same goes for VGA test
}
  '===============================================================
  'have priority on TV output detection
  HW_Discharge(12, CHKDELAY, %0111)
  HW_PreBias(12, PREBIASLEN, %0111)
  '---------------------------------------------------------------
  'read input 32 times while keeping clk on P14 and pullup on P12
  OUTA[15..12] := %0101
  DIRA[15..12] := %0001
  oval := 0
  tval := 0
  repeat 32
    OUTA[14] := oval
    oval := oval ^ 1
    tval += INA[13]
  '---------------------------------------------------------------
  HW_Discharge(12, CHKDELAY, %0111)
  '---------------------------------------------------------------
  if tval < 16
    'TV set connected => stop here and use TVC
    HW_Release
    return HW_CONIO_TVCKBD
  '===============================================================
  HW_Discharge(20, CHKDELAY, %1111)
  HW_PreBias(20, PREBIASLEN, %1111)
  '---------------------------------------------------------------
  'read input 32 times while keeping clk on P14 and pullup on P12
  OUTA[23..20] := %1010
  DIRA[23..20] := %1010
  oval := %0101
  tval := 0
  repeat 32
    OUTA[23..20] := oval
    oval := oval ^ %1111
    tval += (INA[22] & INA[20])
  '---------------------------------------------------------------
  HW_Discharge(20, CHKDELAY, %0111)
  '---------------------------------------------------------------
  if tval < 16
    'VGA cable connected => stop here and use VGA
    HW_Release
    return HW_CONIO_VGAKBD
  '===============================================================
  HW_Release
  return HW_CONIO_SERIAL

PRI AutoDetect_Proto | rval, tval, oval
{
  Auto detect for ProtoBoard (and DracBlade)
  1. detect TVA adapter at P16-P19 => use TV  output at P16-P18
  2. detect VGA monitor at P16-P19 => use VGA output at P16-P23
  3. fallback if no video          => use std serial at P30,P31
}
  '---------------------------------------------------------------
  HW_Discharge(16, CHKDELAY, %1111)
  HW_PreBias(16, PREBIASLEN, %1111)
  '---------------------------------------------------------------
  tval := 0
  DIRA[19..16] := %1100
  OUTA[19..16] := %1100
  tval := (tval << 4) | INA[19..16]
  OUTA[19..16] := %1101
  tval := (tval << 4) | INA[19..16]
  DIRA[19..16] := %1101
  OUTA[19..16] := %1100
  tval := (tval << 4) | INA[19..16]
  OUTA[19..16] := %1101
  tval := (tval << 4) | INA[19..16]
  '---------------------------------------------------------------
  HW_Discharge(16, CHKDELAY, %1111)
  '---------------------------------------------------------------
  case tval
    $EEEF: 'disconnected (but VGA circuit present) => fallback to serial port
      rval := HW_CONIO_SERIAL
    $CCCD: 'VGA monitor connected, will use that for output
      rval := HW_CONIO_VGAKBD
    $FFEF: 'external TVA adaper connected, assume we want to use TV
      rval := HW_CONIO_TVCKBD
    other: 'unknown => error, unable to do autodetection => do other action (?)
      rval := HW_CONIO_FAILED
  '---------------------------------------------------------------
  HW_Release
  return rval

PRI AutoDetect_RamBlade | rval, tval, oval
{
  Auto detect for RamBlade (in stand-alone mode)
  1. detect TV connected at P22,P23 => use 1pin TV & 1pin KB at P22,P23
  2. fallback if no video           => use serial port at P22,P23
}
  '---------------------------------------------------------------
  HW_Discharge(20, CHKDELAY, %0100) 'only 1 pin here, want to avoid contention
  HW_PreBias(20, PREBIASLEN, %0100) 'with directly connected serial link
  '---------------------------------------------------------------
  'read input 32 times after some conditioning
  tval := 0
  repeat 32
    HW_PreBias(20, RBBURSTLEN, %0100) 'balanced bias, redo prebias for each bit
    DIRA[22] := 1
    repeat RBBURSTLEN '\
      OUTA[22] := 0   '|- slight positive bias due to loop!
      OUTA[22] := 1   '/
    DIRA[22] := 0
    tval := (tval << 1) | INA[22]
  '---------------------------------------------------------------
  HW_Discharge(20, CHKDELAY, %0100)
  oval := 0
  repeat 32
    if (tval & 1) == 1
      oval := oval + 1
  if oval < 16
    'TV detected (for AC stimulus, pin was kept low by TV low inpedance)
    rval := HW_CONIO_1PINVK
  else
    'default to serial link (pin remain high vs serial rx on other side)
    rval := HW_CONIO_SERIAL
  '---------------------------------------------------------------
  HW_Release
  return rval

PRI AutoDetect_Unknown | rval, dval, tval, oval
  'unknown is unknown, dude!
  HW_Release
  return HW_CONIO_FAILED

CON
{{
┌──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┐
│                                                   TERMS OF USE: MIT License                                                  │
├──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤
│Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    │
│files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    │
│modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software│
│is furnished to do so, subject to the following conditions:                                                                   │
│                                                                                                                              │
│The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.│
│                                                                                                                              │
│THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          │
│WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         │
│COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   │
│ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         │
└──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘
}}

